from model_munger.model import Location, ModelType

SNARK = ModelType(
    id="snark",
    short_name="SNARK",
    full_name="System for Numerical Atmospheric Research and Kinetics (SNARK)",
)
HELSINKI = Location(id="helsinki", name="Helsinki")
